package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.Timestep;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.CompositeCountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.utils.LogWriter;

public abstract class AbstractDemandManager {

	protected CompositeCountryManager compositeCountryManager;
	protected BioenergyDemandManager bioenergyDemandManager;
	protected CerealFractionsManager cerealFractionsManager;

	public AbstractDemandManager(CompositeCountryManager compositeCountryManager) {
		this.compositeCountryManager = compositeCountryManager;
		bioenergyDemandManager = new BioenergyDemandManager();
		cerealFractionsManager = new CerealFractionsManager(compositeCountryManager);
	}

	public Map<CommodityType, Double> getDemand(CompositeCountry cc, int year) {

		if (!ModelConfig.CHANGE_DEMAND_YEAR)
			year = ModelConfig.BASE_YEAR;

		Map<CommodityType, Double> demandMap = new HashMap<CommodityType, Double>();
		Map<CommodityType, Double> foodDemandMap;

		for (SingleCountry c : compositeCountryManager.getAllForCompositeCountry(cc)) {

			foodDemandMap = getFoodDemand(c, year);

			for (CommodityType commodity : CommodityType.getAllItems()) {

				Double food = foodDemandMap.get(commodity);
				double bioenergy = bioenergyDemandManager.getFirstGenBioenergyDemand(c, year, commodity);
				// LogWriter.println(String.format("Country %s comm %s: %f",
				// country, commodity, bioenergy));
				
				double newDemand = ((food == null ? 0.0 : food.doubleValue()) + bioenergy)
						* (1.0 + commodity.getSeedWasteRate());
				Double prevDemand = demandMap.get(commodity);

				if (prevDemand != null)
					newDemand += prevDemand.doubleValue(); // aggregate if
															// composite country

				demandMap.put(commodity, newDemand);
			}
		}
		
		return demandMap;
	}

	abstract Map<CommodityType, Double> getFoodDemand(SingleCountry c, int year);

	public double getSecondGenBioenergyDemand(Timestep timestep) {
		int year = ModelConfig.CHANGE_DEMAND_YEAR ? timestep.getYear() : ModelConfig.BASE_YEAR;
		double d = bioenergyDemandManager.getSecondGenBioenergyDemand(year);
		LogWriter.println("Global gen2 bioenergy demand in " + timestep + " is " + d);
		return d;
	}
	
	public Map<CropType, Double> getBaseCerealFraction(CompositeCountry cc) {
		return cerealFractionsManager.getBaseCerealFraction(cc);
	}

	public Map<CommodityType, Double> dietaryAdjustment(Map<CommodityType, Double> originalFoodDemandMap, int year) {
		
		int yearsOfChange = Math.min(ModelConfig.DIETARY_CHANGE_END_YEAR - ModelConfig.DIETARY_CHANGE_START_YEAR,
				year - ModelConfig.DIETARY_CHANGE_START_YEAR);
		double monoDemandAdj = yearsOfChange > 0 ? (yearsOfChange * CommodityType.MONOGASTRICS.demandChangeRate()) : 0.0;
		double rumDemandAdj = yearsOfChange > 0 ? (yearsOfChange * CommodityType.RUMINANTS.demandChangeRate()) : 0.0;

		Double monogastricDemand = originalFoodDemandMap.get(CommodityType.MONOGASTRICS);
		double monogastricEnergyShortfall = (monogastricDemand != null)
				? monogastricDemand * monoDemandAdj * CommodityType.MONOGASTRICS.getEJPerMT() : 0.0;

		Double ruminantDemand = originalFoodDemandMap.get(CommodityType.RUMINANTS);
		double ruminantsEnergyShortfall = (ruminantDemand != null)
				? ruminantDemand * rumDemandAdj * CommodityType.RUMINANTS.getEJPerMT() : 0.0;

		Map<CommodityType, Double> updatedFoodDemandMap = new HashMap<CommodityType, Double>();

		for (CommodityType commodity : CommodityType.getAllItems()) {
			Double originalCommodityDemand = originalFoodDemandMap.get(commodity);

			if (originalCommodityDemand != null) {
				if (commodity.isAnimalProduct()) {
					double adj = yearsOfChange > 0 ? (yearsOfChange * commodity.demandChangeRate()) : 0.0;
					updatedFoodDemandMap.put(commodity,
							originalCommodityDemand*(1-adj));
				} else {
					updatedFoodDemandMap.put(commodity,
							originalCommodityDemand
									+ commodity.getMeatSubstitutionProportion() * ruminantsEnergyShortfall
											/ commodity.getEJPerMT()
									+ commodity.getMeatSubstitutionProportion() * monogastricEnergyShortfall
											/ commodity.getEJPerMT());

				}
			}

		}
		
//		LogWriter.println("mono demand adj " + monoDemandAdj +" rum demand adj " + rumDemandAdj + " year " + year + " years of change " + yearsOfChange + " mono shortfall " + monogastricEnergyShortfall + "ruminant shortfall " +  ruminantsEnergyShortfall);
//		LogWriter.println("original map " + originalFoodDemandMap);
//		LogWriter.println("updated map " + updatedFoodDemandMap);
		return updatedFoodDemandMap;
	}

}
